@@ -1,5 +1,5 @@
 package Yancy;
-our $VERSION = '1.023';
+our $VERSION = '1.024';
 # ABSTRACT: A simple framework and editor for content-driven Mojolicious websites
 
 #pod =head1 SYNOPSIS
@@ -195,7 +195,7 @@ Yancy - A simple framework and editor for content-driven Mojolicious websites
 
 =head1 VERSION
 
-version 1.023
+version 1.024
 
 =head1 SYNOPSIS
 
@@ -349,7 +349,7 @@ Doug Bell <preaction@cpan.org>
 
 =head1 CONTRIBUTORS
 
-=for stopwords Ed J Mohammad S Anwar William Lindley
+=for stopwords Ed J Mohammad S Anwar Rajesh Mallah William Lindley
 
 =over 4
 
@@ -363,6 +363,10 @@ Mohammad S Anwar <mohammad.anwar@yahoo.com>
 
 =item *
 
+Rajesh Mallah <mallah.rajesh@gmail.com>
+
+=item *
+
 William Lindley <wlindley@wlindley.com>
 
 =back
